@@ -116,6 +116,7 @@ struct _BattfinkApmPriv {
 enum {
   BATTFINK_SIG_PREFERENCE,
   BATTFINK_SIG_DISABLE,
+  BATTFINK_SIG_DISABLE_LABEL,
   BATTFINK_SIG_LAST
 };
 
@@ -243,9 +244,9 @@ battfink_apm_update_tooltip (BattfinkApm *battfink_apm)
 
   time = battfink_apm_get_time (battfink_apm);
 
-  if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage != 100) 
+  if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_charging) 
     tip = g_strdup_printf (_("Charging: %s"), time);
-  else if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage == 100)
+  else if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_charging)
     tip = g_strdup_printf (_("System is running on AC power"));
   else
     tip = g_strdup_printf (_("Battery: %s"), time);
@@ -299,10 +300,10 @@ battfink_apm_update_image (BattfinkApm *battfink_apm)
 
   battfink_apm_update_tooltip (battfink_apm);
 
-  /* If the power is on and battery is fully charged, then remove the timeout and show the 
+  /* If the power is on and battery is not charging, then remove the timeout and show the 
    * power icon
    */
-  if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage == 100) {
+  if (battfink_apm->priv->ac_power && !battfink_apm->priv->battery_charging) {
 
     /* remove the timeout for animation if neccessary */
     if (battfink_apm->priv->charging_timeout != -1) {
@@ -314,11 +315,11 @@ battfink_apm_update_image (BattfinkApm *battfink_apm)
     return TRUE;
   }
 
-  /* If the power is on and the battery is not fully charged, add the timeout
+  /* If the power is on and the battery is charging, add the timeout
    */
   if (battfink_apm->priv->charging_timeout == -1 && 
       battfink_apm->priv->ac_power && 
-      battfink_apm->priv->battery_percentage < 100) {
+      battfink_apm->priv->battery_charging) {
 
     if (!first_time) {
       /* This is a pretty ugly hack to get things to work properly */
@@ -370,11 +371,34 @@ battfink_apm_enable (BattfinkApm *battfink_apm, EggTrayIcon *tray_icon)
     gtk_image_set_from_pixbuf (GTK_IMAGE (battfink_apm->priv->image), battfink_apm->priv->battery_icon[BATTFINK_APM_POWER]);
     gtk_tooltips_set_tip (GTK_TOOLTIPS (battfink_apm->priv->tooltips), GTK_WIDGET (battfink_apm), 
                           _("System is running on AC power"), NULL);
+    
     gtk_widget_show_all (GTK_WIDGET (tray_icon));
+    gtk_widget_hide (battfink_apm->priv->label);
     return;
   }
 
   battfink_apm_update_image (battfink_apm);
+
+  gtk_widget_show_all (GTK_WIDGET (tray_icon));
+  gtk_widget_hide (battfink_apm->priv->label);
+
+  if (battfink_apm->priv->timeout == -1)
+    battfink_apm->priv->timeout = g_timeout_add (1000, (GSourceFunc) battfink_apm_update_image, battfink_apm);
+}
+
+static void
+battfink_apm_enable_label (BattfinkApm *battfink_apm, EggTrayIcon *tray_icon)
+{
+  
+  if (battfink_apm->priv->battery_flags & BATTERY_FLAGS_ABSENT) {
+    gtk_image_set_from_pixbuf (GTK_IMAGE (battfink_apm->priv->image), battfink_apm->priv->battery_icon[BATTFINK_APM_POWER]);
+    gtk_tooltips_set_tip (GTK_TOOLTIPS (battfink_apm->priv->tooltips), GTK_WIDGET (battfink_apm), 
+                          _("System is running on AC power"), NULL);
+    
+    gtk_widget_show_all (GTK_WIDGET (tray_icon));
+    return;
+  }
+
   battfink_apm_update_label (battfink_apm);
 
   gtk_widget_show_all (GTK_WIDGET (tray_icon));
@@ -394,7 +418,13 @@ battfink_apm_disable (BattfinkApm *battfink_apm, EggTrayIcon *tray_icon)
     g_source_remove (battfink_apm->priv->charging_timeout);
 
   battfink_apm->priv->charging_timeout = -1;
-  battfink_apm->priv->timeout = -1; 
+  battfink_apm->priv->timeout = -1;
+}
+
+static void
+battfink_apm_disable_label (BattfinkApm *battfink_apm, EggTrayIcon *tray_icon)
+{
+  gtk_widget_hide (battfink_apm->priv->label);
 }
 
 static gint
@@ -551,6 +581,12 @@ battfink_apm_remove_icon_callback (GtkWidget *widget, gpointer user_data)
   g_signal_emit (G_OBJECT (user_data), battfink_apm_signals [BATTFINK_SIG_DISABLE], 0);
 }
 
+static void
+battfink_apm_remove_label_callback (GtkWidget *widget, gpointer user_data)
+{
+  g_signal_emit (G_OBJECT (user_data), battfink_apm_signals [BATTFINK_SIG_DISABLE_LABEL], 0);
+}
+
 static void
 battfink_apm_open_preferences (GtkWidget *widget, gpointer user_data)
 {
@@ -697,6 +733,8 @@ battfink_apm_class_init (BattfinkApmClass *klass)
 
   klass->enable  = battfink_apm_enable;
   klass->disable = battfink_apm_disable;
+  klass->enable_label = battfink_apm_enable_label;
+  klass->disable_label = battfink_apm_disable_label;
   klass->time_left = battfink_apm_time_left;
 
   widget_class->button_press_event = battfink_apm_button_press_event;
@@ -716,6 +754,14 @@ battfink_apm_class_init (BattfinkApmClass *klass)
                                                                NULL, NULL,
                                                                g_cclosure_marshal_VOID__VOID,
                                                                G_TYPE_NONE, 0);
+
+  battfink_apm_signals [BATTFINK_SIG_DISABLE] = g_signal_new ("disable_label", 
+                                                               G_OBJECT_CLASS_TYPE (gobject_class), 
+                                                               G_SIGNAL_RUN_FIRST, 
+                                                               G_STRUCT_OFFSET (BattfinkApmClass, label_disable),
+                                                               NULL, NULL,
+                                                               g_cclosure_marshal_VOID__VOID,
+                                                               G_TYPE_NONE, 0);
 }
 
 static void
